@@ -23,16 +23,12 @@
 
 #include "linklist.h"
 #include "memory.h"
-
+
 /* Allocate new list. */
 struct list *
 list_new (void)
 {
-  struct list *new;
-
-  new = XMALLOC (MTYPE_LINK_LIST, sizeof (struct list));
-  memset (new, 0, sizeof (struct list));
-  return new;
+  return XCALLOC (MTYPE_LINK_LIST, sizeof (struct list));
 }
 
 /* Free list. */
@@ -46,11 +42,7 @@ list_free (struct list *l)
 static struct listnode *
 listnode_new (void)
 {
-  struct listnode *node;
-
-  node = XMALLOC (MTYPE_LINK_NODE, sizeof (struct listnode));
-  memset (node, 0, sizeof (struct listnode));
-  return node;
+  return XCALLOC (MTYPE_LINK_NODE, sizeof (struct listnode));
 }
 
 /* Free listnode. */
@@ -59,7 +51,7 @@ listnode_free (struct listnode *node)
 {
   XFREE (MTYPE_LINK_NODE, node);
 }
-
+
 /* Add new data to the list. */
 void
 listnode_add (struct list *list, void *val)
@@ -250,7 +242,7 @@ listnode_lookup (struct list *list, void *data)
       return node;
   return NULL;
 }
-
+
 /* Delete the node from list.  For ospfd and ospf6d. */
 void
 list_delete_node (struct list *list, struct listnode *node)
@@ -266,7 +258,7 @@ list_delete_node (struct list *list, struct listnode *node)
   list->count--;
   listnode_free (node);
 }
-
+
 /* ospf_spf.c */
 void
 list_add_node_prev (struct list *list, struct listnode *current, void *val)
