@@ -27,7 +27,7 @@
 unsigned long rip_debug_event = 0;
 unsigned long rip_debug_packet = 0;
 unsigned long rip_debug_zebra = 0;
-
+
 DEFUN (show_debugging_rip,
        show_debugging_rip_cmd,
        "show debugging rip",
@@ -44,19 +44,16 @@ DEFUN (show_debugging_rip,
     {
       if (IS_RIP_DEBUG_SEND && IS_RIP_DEBUG_RECV)
 	{
-	  vty_out (vty, "  RIP packet%s debugging is on%s",
-		   IS_RIP_DEBUG_DETAIL ? " detail" : "",
+	  vty_out (vty, "  RIP packet debugging is on%s",
 		   VTY_NEWLINE);
 	}
       else
 	{
 	  if (IS_RIP_DEBUG_SEND)
-	    vty_out (vty, "  RIP packet send%s debugging is on%s",
-		     IS_RIP_DEBUG_DETAIL ? " detail" : "",
+	    vty_out (vty, "  RIP packet send debugging is on%s",
 		     VTY_NEWLINE);
 	  else
-	    vty_out (vty, "  RIP packet receive%s debugging is on%s",
-		     IS_RIP_DEBUG_DETAIL ? " detail" : "",
+	    vty_out (vty, "  RIP packet receive debugging is on%s",
 		     VTY_NEWLINE);
 	}
     }
@@ -105,11 +102,12 @@ DEFUN (debug_rip_packet_direct,
     rip_debug_packet |= RIP_DEBUG_SEND;
   if (strncmp ("recv", argv[0], strlen (argv[0])) == 0)
     rip_debug_packet |= RIP_DEBUG_RECV;
-  rip_debug_packet &= ~RIP_DEBUG_DETAIL;
   return CMD_SUCCESS;
 }
 
-DEFUN (debug_rip_packet_detail,
+/* N.B. the "detail" modifier is a no-op.  we leave this command
+   for legacy compatibility. */
+DEFUN_DEPRECATED (debug_rip_packet_detail,
        debug_rip_packet_detail_cmd,
        "debug rip packet (recv|send) detail",
        DEBUG_STR
@@ -124,7 +122,6 @@ DEFUN (debug_rip_packet_detail,
     rip_debug_packet |= RIP_DEBUG_SEND;
   if (strncmp ("recv", argv[0], strlen (argv[0])) == 0)
     rip_debug_packet |= RIP_DEBUG_RECV;
-  rip_debug_packet |= RIP_DEBUG_DETAIL;
   return CMD_SUCCESS;
 }
 
@@ -203,7 +200,7 @@ DEFUN (no_debug_rip_zebra,
 }
 
 /* Debug node. */
-struct cmd_node debug_node =
+static struct cmd_node debug_node =
 {
   DEBUG_NODE,
   "",				/* Debug node has no interface. */
@@ -224,20 +221,17 @@ config_write_debug (struct vty *vty)
     {
       if (IS_RIP_DEBUG_SEND && IS_RIP_DEBUG_RECV)
 	{
-	  vty_out (vty, "debug rip packet%s%s",
-		   IS_RIP_DEBUG_DETAIL ? " detail" : "",
+	  vty_out (vty, "debug rip packet%s",
 		   VTY_NEWLINE);
 	  write++;
 	}
       else
 	{
 	  if (IS_RIP_DEBUG_SEND)
-	    vty_out (vty, "debug rip packet send%s%s",
-		     IS_RIP_DEBUG_DETAIL ? " detail" : "",
+	    vty_out (vty, "debug rip packet send%s",
 		     VTY_NEWLINE);
 	  else
-	    vty_out (vty, "debug rip packet recv%s%s",
-		     IS_RIP_DEBUG_DETAIL ? " detail" : "",
+	    vty_out (vty, "debug rip packet recv%s",
 		     VTY_NEWLINE);
 	  write++;
 	}
