@@ -38,7 +38,7 @@
 
 #include "ripngd/ripngd.h"
 #include "ripngd/ripng_debug.h"
-
+
 /* If RFC2133 definition is used. */
 #ifndef IPV6_JOIN_GROUP
 #define IPV6_JOIN_GROUP  IPV6_ADD_MEMBERSHIP 
@@ -157,22 +157,6 @@ ripng_if_ipv6_lladdress_check (struct interface *ifp)
   return count;
 }
 
-/* Check max mtu size. */
-static unsigned int
-ripng_check_max_mtu (void)
-{
-  struct listnode *node;
-  struct interface *ifp;
-  unsigned int mtu;
-
-  mtu = 0;
-  for (ALL_LIST_ELEMENTS_RO (iflist, node, ifp))
-    if (mtu < ifp->mtu6)
-      mtu = ifp->mtu6;
-
-  return mtu;
-}
-
 static int
 ripng_if_down (struct interface *ifp)
 {
@@ -243,8 +227,9 @@ ripng_interface_up (int command, struct zclient *zclient, zebra_size_t length)
     return 0;
 
   if (IS_RIPNG_DEBUG_ZEBRA)
-    zlog_debug ("interface up %s index %d flags %ld metric %d mtu %d",
-	       ifp->name, ifp->ifindex, ifp->flags, ifp->metric, ifp->mtu6);
+    zlog_debug ("interface up %s index %d flags %llx metric %d mtu %d",
+	       ifp->name, ifp->ifindex, (unsigned long long)ifp->flags,
+	       ifp->metric, ifp->mtu6);
 
   /* Check if this interface is RIPng enabled or not. */
   ripng_enable_apply (ifp);
@@ -276,8 +261,9 @@ ripng_interface_down (int command, struct zclient *zclient,
   ripng_if_down (ifp);
 
   if (IS_RIPNG_DEBUG_ZEBRA)
-    zlog_debug ("interface down %s index %d flags %ld metric %d mtu %d",
-	       ifp->name, ifp->ifindex, ifp->flags, ifp->metric, ifp->mtu6);
+    zlog_debug ("interface down %s index %d flags %#llx metric %d mtu %d",
+		ifp->name, ifp->ifindex,
+		(unsigned long long) ifp->flags, ifp->metric, ifp->mtu6);
 
   return 0;
 }
@@ -291,8 +277,9 @@ ripng_interface_add (int command, struct zclient *zclient, zebra_size_t length)
   ifp = zebra_interface_add_read (zclient->ibuf);
 
   if (IS_RIPNG_DEBUG_ZEBRA)
-    zlog_debug ("RIPng interface add %s index %d flags %ld metric %d mtu %d",
-	       ifp->name, ifp->ifindex, ifp->flags, ifp->metric, ifp->mtu6);
+    zlog_debug ("RIPng interface add %s index %d flags %#llx metric %d mtu %d",
+		ifp->name, ifp->ifindex, (unsigned long long) ifp->flags,
+		ifp->metric, ifp->mtu6);
 
   /* Check is this interface is RIP enabled or not.*/
   ripng_enable_apply (ifp);
@@ -324,8 +311,9 @@ ripng_interface_delete (int command, struct zclient *zclient,
     ripng_if_down(ifp);
   }
 
-  zlog_info("interface delete %s index %d flags %ld metric %d mtu %d",
-            ifp->name, ifp->ifindex, ifp->flags, ifp->metric, ifp->mtu6);
+  zlog_info("interface delete %s index %d flags %#llx metric %d mtu %d",
+            ifp->name, ifp->ifindex, (unsigned long long) ifp->flags,
+	    ifp->metric, ifp->mtu6);
 
   /* To support pseudo interface do not free interface structure.  */
   /* if_delete(ifp); */
@@ -517,7 +505,7 @@ ripng_interface_address_delete (int command, struct zclient *zclient,
 
   return 0;
 }
-
+
 /* RIPng enable interface vector. */
 vector ripng_enable_if;
 
@@ -824,7 +812,7 @@ ripng_enable_apply_all (void)
   for (ALL_LIST_ELEMENTS_RO (iflist, node, ifp))
     ripng_enable_apply (ifp);
 }
-
+
 /* Clear all network and neighbor configuration */
 void
 ripng_clean_network ()
@@ -847,7 +835,7 @@ ripng_clean_network ()
       vector_slot (ripng_enable_if, i) = NULL;
     }
 }
-
+
 /* Vector to store passive-interface name. */
 vector Vripng_passive_interface;
 
@@ -1115,7 +1103,7 @@ DEFUN (no_ripng_passive_interface,
 {
   return ripng_passive_interface_unset (vty, argv[0]);
 }
-
+
 static struct ripng_interface *
 ri_new (void)
 {
