@@ -358,6 +358,9 @@ zebra_interface_address_add_update (struct interface *ifp,
 		  p->prefixlen, ifc->ifp->name);
     }
 
+  if (!CHECK_FLAG(ifc->conf, ZEBRA_IFC_REAL))
+    zlog_warn("WARNING: advertising address to clients that is not yet usable.");
+
   router_id_add_address(ifc);
 
   for (ALL_LIST_ELEMENTS (zebrad.client_list, node, nnode, client))
